﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace Drawing
{
    public partial class Form1 : Form
    {
        private Graphics g;
        private Bitmap bitmap = new Bitmap(SystemInformation.PrimaryMonitorMaximizedWindowSize.Width, SystemInformation.PrimaryMonitorMaximizedWindowSize.Height);

        private GraphicsPath curve = new GraphicsPath();
        private Point prevPos, currPos;

        private int mode = 0; // 0 - free draw, 1 - rubber band line
        
        public Form1()
        {
            InitializeComponent();

            // double buffering
            SetStyle(ControlStyles.UserPaint, true);
            SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            SetStyle(ControlStyles.DoubleBuffer, true);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            g = Graphics.FromImage(bitmap);
            g.Clear(this.BackColor);
            RefreshBackground();
        }

        private void RefreshBackground()
        {
            Size sz = this.Size;
            Rectangle rect = new Rectangle(0, 0, sz.Width, sz.Height);
            this.BackgroundImage = bitmap.Clone(rect, bitmap.PixelFormat);
        }

        private void Form1_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                prevPos = new Point(e.X, e.Y);

                if (mode == 0)
                    curve.StartFigure();
            }
        }

        private void Form1_MouseMove(object sender, MouseEventArgs e)
        {
            currPos = new Point(e.X, e.Y);

            if (Capture && e.Button == MouseButtons.Left)
            {
                if (mode == 0)
                {
                    curve.AddLine(prevPos, currPos);
                    g.DrawLine(Pens.Black, prevPos, currPos);
                    prevPos = currPos;
                }
                Invalidate();
            }
        }

        private void Form1_MouseUp(object sender, MouseEventArgs e)
        {
            currPos = new Point(e.X, e.Y);

            if (mode == 0)
            {
                curve.AddLine(prevPos, currPos);
                g.DrawLine(Pens.Black, prevPos, currPos);
                prevPos = currPos;
            }
            else if (mode == 1)
            {
                g.DrawLine(Pens.Black, prevPos.X, prevPos.Y, currPos.X, currPos.Y);
            }

            RefreshBackground();
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            if (Capture)
            {
                if (mode == 0)
                    e.Graphics.DrawPath(Pens.Black, curve);
                else if (mode == 1)
                    e.Graphics.DrawLine(Pens.Black, prevPos.X, prevPos.Y, currPos.X, currPos.Y);
            }
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            mode = 0;
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            mode = 1;
        }

        private void Form1_SizeChanged(object sender, EventArgs e)
        {
            RefreshBackground();
        }
    }
}